;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;An instrument is a name (a symbol) bound to a function. When called,
;;the function should return a synthesis network.
;;
;;An orchestra is an association between instrument names and graph
;;creation functions.
;;
;;; Code:

(define-module (soundscrape orchestras))

(define-public (make-instrument name proc)
"Create an instrument named @var{name} with a graph creation function
@var{proc}."
  (if (not (symbol? name)) (error "Bad name (must be a symbol):" name))
  (if (not (procedure? proc)) (error "Bad procedure:" proc))
  (cons name proc))

(define-public (make-orchestra . instruments)
  "Create an orchestra, adding all arguments as instruments."
  (let ((orc (make-hash-table 31)))
    (for-each
     (lambda (pair) (hashq-create-handle! orc (car pair) (cdr pair)))
     instruments)
    orc))

(define-public (orchestra-ref orc name)
  "Retrieve the graph creation function for the instrument named
@var{name} from orchestra @var{orc}."
  (let ((val (hashq-ref orc name #f)))
    (if (not val) (error "No such instrument" name))
    val))

(define-public orchestra-fold hash-fold)
