/* Soundscrape
 * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
 *
 * ss-element.c: Subclass of GstElement supporting both control- and 
 * audio-rate data
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 */


#include <config.h>
#include "ss.h"


enum {
  /* FILL ME */
  LAST_SIGNAL
};

enum {
  PROP_0,
  PROP_SS_RATE
};


static void ss_element_class_init	(SSElementClass *klass);
static void ss_element_set_property	(GObject *object, guint prop_id,
                                         const GValue *value, GParamSpec *pspec);
static void ss_element_get_property	(GObject *object, guint prop_id,
                                         GValue *value, GParamSpec *pspec);

static GstElementStateReturn ss_element_change_state	(GstElement *element);

static GstElementClass *parent_class = NULL;


SS_TYPE_MAKE_BOILERPLATE (ss_element_get_type, GST_TYPE_ELEMENT,
                          "SSElement", SSElementClass, SSElement,
                          ss_element_class_init, NULL /* abstract type */);

static GType
ss_element_rate_get_type (void)
{
  static GType etype = 0;

  if (etype == 0) {
    static const GEnumValue values[] = {
      { SS_ELEMENT_RATE_AUDIO, "SS_ELEMENT_RATE_AUDIO", "audio" },
      { SS_ELEMENT_RATE_CONTROL, "SS_ELEMENT_RATE_CONTROL", "control" },
      { 0, NULL, NULL }
    };
    etype = g_enum_register_static ("SSElementRate", values);
  }
  return etype;
}

static void 
ss_element_class_init (SSElementClass *klass) 
{
  GObjectClass *gobject_class;
  GParamSpec *pspec;

  gobject_class = (GObjectClass*)klass;

  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);

  pspec = g_param_spec_enum ("ss-rate", "Output rate",
                             "Either audio rate or control rate",
                             SS_TYPE_ELEMENT_RATE, SS_ELEMENT_RATE_AUDIO,
                             G_PARAM_READWRITE);
  g_object_class_install_property (gobject_class, PROP_SS_RATE, pspec);

  gobject_class->set_property = ss_element_set_property;  
  gobject_class->get_property = ss_element_get_property;
  GST_ELEMENT_CLASS (klass)->change_state = ss_element_change_state;
}

static void 
ss_element_set_property (GObject *object, guint prop_id,
                         const GValue *value, GParamSpec *pspec) 
{
  SSElement *element = SS_ELEMENT (object);

  switch (prop_id) {
    case PROP_SS_RATE:
      element->ss_rate = g_value_get_enum (value);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static void
ss_element_get_property (GObject *object, guint prop_id,
                         GValue *value, GParamSpec *pspec)
{
  SSElement *element = SS_ELEMENT (object);

  switch (prop_id) {
    case PROP_SS_RATE:
      g_value_set_enum (value, element->ss_rate);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static GstElementStateReturn
ss_element_change_state (GstElement *element)
{
    SSElementClass *klass = SS_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));

    switch (GST_STATE_TRANSITION (element)) {
    case GST_STATE_READY_TO_NULL:
        if (klass->cleanup)
            klass->cleanup (element);
        break;
    default:
        break;
    }

    if (parent_class->change_state)
        return parent_class->change_state (element);

    return GST_STATE_SUCCESS;
}
