/* Soundscrape
 * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
 *
 * ss-element.h: Subclass of GstElement supporting both control- and 
 * audio-rate data
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 */

#ifndef __SS_ELEMENT_H__
#define __SS_ELEMENT_H__


#include <gst/gst.h>
#include <sys/types.h>

G_BEGIN_DECLS

#define SS_TYPE_ELEMENT			(ss_element_get_type())
#define SS_ELEMENT_CAST(obj)		((SSElement*)(obj))
#define SS_ELEMENT_CLASS_CAST(obj)	((SSElementClass*)(obj))
#define SS_IS_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj),SS_TYPE_ELEMENT))
#define SS_IS_ELEMENT_CLASS(obj)	(G_TYPE_CHECK_CLASS_TYPE((klass),SS_TYPE_ELEMENT))

#ifdef SS_TYPE_PARANOID
# define SS_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), SS_TYPE_ELEMENT, SSElement))
# define SS_ELEMENT_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), SS_TYPE_ELEMENT, SSElementClass))
#else
# define SS_ELEMENT			SS_ELEMENT_CAST
# define SS_ELEMENT_CLASS		SS_ELEMENT_CLASS_CAST
#endif

#define SS_TYPE_ELEMENT_RATE (ss_element_rate_get_type())
#define SS_ELEMENT_RATE(obj) (SS_ELEMENT (obj)->ss_rate)
#define SS_ELEMENT_IS_RATE(obj,which) (SS_ELEMENT_RATE (obj)==SS_ELEMENT_RATE_##which)


typedef enum {
    SS_ELEMENT_FLAG_LAST = GST_ELEMENT_FLAG_LAST + 2,
} SSElementFlags;

typedef struct _SSElement SSElement;
typedef struct _SSElementClass SSElementClass;

typedef enum {
    SS_ELEMENT_RATE_AUDIO,
    SS_ELEMENT_RATE_CONTROL
} SSElementRate;


struct _SSElement {
    GstElement element;
    
    SSElementRate ss_rate;

    gint rate, buffer_frames;
};

struct _SSElementClass {
    GstElementClass parent_class;

    void (*cleanup) (GstElement*);
};

GType ss_element_get_type (void);

G_END_DECLS

#endif /* __SS_ELEMENT_H__ */
