/* Soundscrape
 * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
 *
 * ss-multi-op.h: Commutative operations taking a variable number of arguments
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 *
 * Some code based on gstadder.h from gst-plugins:
 * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
 *               2000 Wim Taymans <wim.taymans@chello.be>
 *               2001 Thomas <thomas@apestaart.org>
 */

#ifndef __SS_MULTI_OP_H__
#define __SS_MULTI_OP_H__


#include <gst/gst.h>

#include "ss.h"


G_BEGIN_DECLS;


#define SS_TYPE_MULTI_OP \
  (ss_multi_op_get_type())
#define SS_MULTI_OP(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),SS_TYPE_MULTI_OP,SSMultiOp))
#define SS_MULTI_OP_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),SS_TYPE_MULTI_OP,SSMultiOpClass))
#define SS_IS_MULTI_OP(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),SS_TYPE_MULTI_OP))
#define SS_IS_MULTI_OP_CLASS(obj) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),SS_TYPE_MULTI_OP))


typedef struct _SSMultiOp             SSMultiOp;
typedef struct _SSMultiOpClass        SSMultiOpClass;


struct _SSMultiOp {
    SSElement	element;

    GstPad	*srcpad;
  
    /* keep track of the sinkpads */
    guint	 numsinkpads;
    GList	*sinkpads;

    gfloat 	(*do_op_c) (gfloat new);
    void 	(*do_op_a) (gfloat *existing, gint how_many);
    gfloat 	(*do_op_cc) (gfloat existing, gfloat new);
    void 	(*do_op_aa) (gfloat *existing, gfloat *new, gint how_many);
    void 	(*do_op_ac) (gfloat *existing, gfloat new, gint how_many);
};

struct _SSMultiOpClass {
    SSElementClass parent_class;
};


GType    ss_multi_op_get_type (void);
gboolean init_ss_multi_op (GstPlugin *plugin);


G_END_DECLS;


#endif /* __SS_PLUS_H__ */
