/* Soundscrape
 * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
 *
 * ss-pad.h: Subclass of GstRealPad supporting default values
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 */


#ifndef __SS_PAD_H__
#define __SS_PAD_H__

#include <gst/gst.h>
#include "ss.h"

G_BEGIN_DECLS

#define SS_TYPE_PAD			(ss_pad_get_type()) 
#define SS_TYPE_PAD_TEMPLATE		(ss_pad_template_get_type()) 

#define SS_PAD_CAST(obj)		((SSPad*)(obj))
#define SS_PAD_CLASS_CAST(klass)	((SSPadClass*)(klass))
#define SS_IS_PAD(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), SS_TYPE_PAD))
#define SS_IS_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), SS_TYPE_PAD))

#ifdef SS_TYPE_PARANOID
# define SS_PAD(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), SS_TYPE_PAD, SSPad))
# define SS_PAD_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), SS_TYPE_PAD, SSPadClass))
#else
# define SS_PAD				SS_PAD_CAST
# define SS_PAD_CLASS			SS_PAD_CLASS_CAST
#endif

#define SS_PAD_VALUE(obj)		(SS_PAD (obj)->value)

#define SS_PAD_TEMPLATE_CAST(obj)	((SSPadTemplate*)(obj))
#define SS_PAD_TEMPLATE_CLASS_CAST(klass) ((SSPadTemplateClass*)(klass))
#define SS_IS_PAD_TEMPLATE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), SS_TYPE_PAD_TEMPLATE))
#define SS_IS_PAD_TEMPLATE_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), SS_TYPE_PAD_TEMPLATE))

#ifdef SS_TYPE_PARANOID
# define SS_PAD_TEMPLATE(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), SS_TYPE_PAD_TEMPLATE, SSPadTemplate))
# define SS_PAD_TEMPLATE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), SS_TYPE_PAD_TEMPLATE, SSPadTemplateClass))
#else
# define SS_PAD_TEMPLATE		SS_PAD_TEMPLATE_CAST
# define SS_PAD_TEMPLATE_CLASS		SS_PAD_TEMPLATE_CLASS_CAST
#endif

#define SS_PAD_TEMPLATE_SRC_FACTORY(name, padname)                              \
static GstPadTemplate*                                                          \
name (void)                                                                     \
{                                                                               \
  static GstPadTemplate *templ = NULL;                                          \
  if (!templ)                                                                   \
    templ = GST_PAD_TEMPLATE (ss_pad_template_src_new (padname));               \
  return templ;                                                                 \
}

#define SS_PAD_TEMPLATE_SINK_FACTORY(name, padname, default_value)              \
static GstPadTemplate*                                                          \
name (void)                                                                     \
{                                                                               \
  static GstPadTemplate *templ = NULL;                                          \
  if (!templ)                                                                   \
    templ = GST_PAD_TEMPLATE (ss_pad_template_sink_new (padname, default_value));\
  return templ;                                                                 \
}

typedef struct _SSPad SSPad;
typedef struct _SSPadClass SSPadClass;
typedef struct _SSPadTemplate SSPadTemplate;
typedef struct _SSPadTemplateClass SSPadTemplateClass;

typedef enum {
    SS_PAD_FLAG_LAST		= GST_PAD_FLAG_LAST + 4
} SSPadFlags;


struct _SSPad {
    GstRealPad 	object;

    gboolean	need_interp;
    gfloat	value;
};

struct _SSPadClass {
    GstRealPadClass parent_class;
};

GstCaps*	ss_fixed_caps_factory		(void);

GType		ss_pad_get_type			(void);
GstPad*		ss_pad_new_from_template	(SSPadTemplate *template);
GstData*	ss_pad_pull			(SSPad *spad, gint buffer_frames);
GstData*	ss_pad_update_control		(SSPad *spad);
void		ss_pad_push			(SSPad *spad, GstData *data);

void		ss_pad_do_add			(SSPad *sadd, GstData **out, gint n);
void		ss_pad_do_mul			(SSPad *smul, GstData **out, gint n);


struct _SSPadTemplate {
    GstPadTemplate	object;

    gfloat		default_value;
};

struct _SSPadTemplateClass {
    GstPadTemplateClass parent_class;
};

GType		ss_pad_template_get_type	(void);
GstPadTemplate*	ss_pad_template_src_new		(const gchar *name);
GstPadTemplate*	ss_pad_template_sink_new	(const gchar *name,
                                                 gfloat default_value);


G_END_DECLS


#endif /* __SS_PAD_H__ */
