/* Soundscrape
 * Copyright (C) 2003 Andy Wingo <wingo at pobox dot com>
 *
 * ss.c: Initialization for the Soundscrape module
 *
 * This program is free software; you can redistribute it and/or    
 * modify it under the terms of the GNU General Public License as   
 * published by the Free Software Foundation; either version 2 of   
 * the License, or (at your option) any later version.              
 *                                                                  
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
 * GNU General Public License for more details.                     
 *                                                                  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact:
 *
 * Free Software Foundation           Voice:  +1-617-542-5942
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 * Boston, MA  02111-1307,  USA       gnu@gnu.org
 */

#include <config.h>

#include "ss.h"

#include "ss-binary-op.h"
#include "ss-enveloptor.h"
#include "ss-ladspa.h"
#include "ss-multi-op.h"
#include "ss-plug.h"
#include "ss-sequencer.h"
#include "ss-unary-op.h"

GstPlugin *ss_plugin;

GType
ss_warp_get_type (void) 
{
    static GType ss_warp_type = 0;
    static GEnumValue ss_warp_type_vals[] = {
        { SS_WARP_LINEAR,	"Linear", "linear"},
        { SS_WARP_EXPONENTIAL, 	"Exponential", "exponential"},
        { SS_WARP_SINE,         "Sine", "sine"},
        { SS_WARP_CURVE,	"Curve (like a discharging capacitor)", "curve"},
        { SS_WARP_WELCH,	"Welch", "welch"},
        { SS_WARP_STEP,		"Step change", "step"},
        { SS_WARP_QUADRATIC,	"Quadratic", "quadratic"},
        { SS_WARP_CUBIC,	"Cubic", "cubic"},
        {0, NULL, NULL},
    };
    if (!ss_warp_type) {
        ss_warp_type = g_enum_register_static ("SSWarpType", ss_warp_type_vals);
    }
    return ss_warp_type;
}

static gboolean
plugin_init (GstPlugin *plugin)
{
    ss_plugin = plugin;

    if (!(init_ss_binary_op (plugin)))
        return FALSE;

    if (!(init_ss_enveloptor (plugin)))
        return FALSE;

    if (!(init_ss_ladspa (plugin)))
        return FALSE;

    if (!(init_ss_multi_op (plugin)))
        return FALSE;

    if (!(init_ss_plug (plugin)))
        return FALSE;

    if (!(init_ss_sequencer (plugin)))
        return FALSE;

    if (!(init_ss_unary_op (plugin)))
        return FALSE;

    ss_pad_get_type ();

    return TRUE;
}

GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "soundscrape",
    "Soundscrape plugins",
    plugin_init,
    VERSION,
    "GPL",
    "Soundscrape",
    "http://ambient.2y.net/soundscrape/"
)
