;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;Different widgets you can use to control your synthesis network.
;;
;;; Code:

(define-module (soundscrape ui controls)
  :use-module (ice-9 optargs)
  :use-module (gnome gtk)
  :use-module (gnome gstreamer)
  :use-module (soundscrape elements) ; for plug
  :use-module (soundscrape util)
  :use-module (soundscrape gw-soundscrape-ui)

  :export (slider-control))

(define (slider-control container . args)
  "Usage: (slider-control container [upper lower value warp])

Create a new slider window and add it to CONTAINER. The return value may
be used within a graph of unit generators:

@example
  (define slider (slider-control (gui-window \"Controls\")))
  (audio sine-faaa :mul (control slider))
@end example

The bounds, default value, and warp method of the slider control may be
set either with positional arguments or with keywords:

@example
 (slider-control (gui-window \"Controls\") :upper 400 :lower 0 :value
20)
@end example

@code{:warp} may be set to either @code{'linear} or @code{'exponential}.
The default is @code{'linear}.
"
  (let-positional-keywords args
                           ((upper 1.0)
                            (lower 0.0)
                            (value 0.0)
                            (warp 'linear))
    (let ((control (make <ssslider-control>
                     :upper upper :lower lower
                     :value value :warp warp)))
      (add container control)
      (show control)
      control)))

(define-method (control (slider <ss-slider-control>))
  (let ((p (control plug)))
    (set slider 'model (get-parent p))
    p))

;; perhaps this should go away?
(define-method (audio (slider <ss-slider-control>))
  (let ((p (audio plug)))
    (set slider 'model (get-parent p))
    p))
