;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;A tree model for the help index.
;;
;;; Code:

(define-module (soundscrape ui help-index)
  :use-module (gnome gtk) ; grr
  :use-module (soundscrape ui filtered-list)
  :use-module (soundscrape util)
  :use-module (soundscrape ui help-tree)
  :use-module (soundscrape ui stext indexing)
  :export (<ss-help-index>)
  :re-export (set-filter))

(define-class-with-docs <ss-help-index> (<ss-filtered-list>)
  "A tree model for the help index.")

(define-method (on-get-n-columns (obj <ss-help-index>))
  3) ;; name, manual name, node name

(define-method (on-get-column-type (obj <ss-help-index>) index)
  (case index
    ((0 1 2) gtype:gchararray)
    (else (error "Invalid index" index))))

(define-method (on-get-value (obj <ss-help-index>) iter index)
  (case index
    ((0 1 2)
     (list-ref iter index))
    (else
     (error "Invalid index" index))))

(define-method (set-filter (obj <ss-help-index>) filter)
  (define (make-index)
    (let loop ((index '()) (iter (iter-nth-child the-help-tree #f 0)))
      (if (not iter)
          (sort! index (lambda rows (apply string-ci<=? (map car rows))))
          (loop (append
                 ((lambda (iter) (if iter (loop '() iter) '()))
                  (iter-nth-child the-help-tree iter 0))
                 (stext-extract-index (get-value the-help-tree iter 1) #f 'all)
                 index)
                (iter-next the-help-tree iter)))))

  (if (null? (slot-ref obj 'list))
      ;; Don't actually do the indexing until the first filter is set,
      ;; that is, until the user has typed something in the search box
      (slot-set! obj 'list (pk (make-index))))
  (next-method))
