;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;Includes unit generators in the help window. For internal use only.
;;
;;; Code:

(define-module (soundscrape ui help-ugens)
  :use-module (srfi srfi-13)
  :use-module (ice-9 documentation)
  :use-module (soundscrape config)
  :use-module (soundscrape elements)
  :use-module (soundscrape util)
  :use-module (soundscrape ui help-tree)
  :use-module (oop goops)
  :use-module (srfi srfi-1)
  :export (find-ugens))

;; need to keep this one up-to-date with any stock gstreamer plugins we
;; include...
(define (category-transformer category)
  (cond
   ((string-prefix? "Soundscrape/" category)
    (substring category 12))
   ((equal? category "Source/Audio")
    "External")
   (else
    category)))

(define ugens-interface (resolve-interface '(soundscrape ugens)))

(define (add-entry top category name class)
  (define (find-category)
    (let loop ((categories (slot-ref top 'children)))
      (cond
       ((null? categories)
        ;; We add a new category
        (let ((new-node (make <ss-help-node>
                          :name category
                          :value `((stext ,category)
                                   "Categories...")
                          :children '()
                          :parent top)))
          (slot-set! top 'children (cons new-node (slot-ref top 'children)))
          new-node))
       ((equal? (slot-ref (car categories) 'name) category)
        (car categories))
       (else
        (loop (cdr categories))))))
  
  (let* ((category (find-category))
         (siblings (slot-ref category 'children))
         (node (make <ss-help-node>
                 :name name
                 :value (cons (list 'stext name)
                              (or
                               (and=> (object-documentation class) cdr)
                               '("[undocumented]")))
                 :parent category)))
    (slot-set! category 'children (cons node siblings))))


(define (find-ugens)
  (let ((top (make <ss-help-node>
               :name "Unit Generators"
               :value '((stext "Unit Generators")
                        (node "top")
                        "Unit generators foo...")
               :children '())))
    (hash-fold
     (lambda (key value accum)
       (add-entry top
                  (category-transformer
                   (get-klass (if (list? value) (car value) value)))
                  (symbol->string key)
                  (module-ref ugens-interface key)))
     #f
     ss-all-ugens)
    top))
