;; Soundscrape
;; Copyright (C) 2003,2004 Andy Wingo <wingo at pobox dot com>

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;The main window of Soundscrape's graphical interface.
;;
;;; Code:

(define-module (soundscrape ui main-window)
  :use-module (gnome gtk)
  ;:use-module (gnome gstreamer)
  ;:use-module (soundscrape util)
  :use-module (soundscrape util)
  :use-module (soundscrape ui util)
  :use-module (soundscrape ui help)
  :use-module (soundscrape ui source-window)
  :use-module (soundscrape ui about-window)
  :use-module (gnome gtk gw-glade)
  :use-module (gnome gtk graphical-repl)

  :export (<ss-gui-main-window>))

(define this-module (current-module))

;;
;; Signal Handlers
;;

(define (on-close widget . args)
  (catch 'cancel close-main-window noop)
  #t)

(define (on-new-source . args)
  (make <ss-gui-source-window>))

(define (on-about . args)
  (show-about-window))

;;
;; Custom Widgets
;;

(define (make-guile-gtk-repl)
  (make <guile-gtk-repl>))

(define-class-with-docs <ss-gui-main-window> ()
  "A class for the main window."
  gtk-window)

(define-method (initialize (obj <ss-gui-main-window>) initargs)
  (define (make-window-menu parent-item)
    (let ((menu (make <gtk-menu>)))
      (let ((item (gtk-image-menu-item-new-with-mnemonic "New _Listener")))
        (set item 'image (make <gtk-image> :stock "gtk-execute"))
        (show item)
        (connect item 'activate unimplemented)
        (gtk-menu-shell-append menu item))
      (let ((item (gtk-image-menu-item-new-with-mnemonic "New Sou_rce")))
        (set item 'image (make <gtk-image> :stock "gtk-new"))
        (show item)
        (connect item 'activate on-new-source)
        (gtk-menu-shell-append menu item))
      (let ((item (make <gtk-separator-menu-item>)))
        (show item)
        (gtk-menu-shell-append menu item))

      (populate-window-menu menu)
      (set-submenu parent-item menu)))

  (if the-main-window
      (errorf "There is already a main window: ~A" the-main-window)
      (set! the-main-window obj))

  (let* ((window (ss-glade-get-widget this-module "soundscrape.glade" "main-window"))
         (grepl (get-widget (glade-get-widget-tree window) "main-repl"))
         (in-port (get grepl 'in-port))
         (out-port (get grepl 'out-port)))

    (add-hook! child-windows-changed-hook
               (lambda ()
                 (make-window-menu
                  (get-widget (glade-get-widget-tree window) "windows"))))

    (slot-set! obj 'gtk-window window)
    (show window)
    (set-current-input-port in-port)
    (set-current-output-port out-port)
    (set-current-error-port out-port)))

(define (close-main-window)
  (for-each close-window child-windows)
  (throw 'quit))
