(define-module (soundscrape ui stext help-nodes)
  :use-module (soundscrape ui help-tree)
  :use-module (soundscrape ui stext)
  :use-module (soundscrape ui stext plain-text)
  :use-module (oop goops)
  :export (stext->help-nodes))

(define (stext->help-nodes stext max-depth)
  "Break @var{stext} into a tree of @code{<ss-help-node>}s. Only break
until sectioning identifiers of depth @var{max-depth}."
  (define (stext->help-nodes-helper stext parent depth)
    (let ((node (make <ss-help-node>
                  :parent parent
                  :name (stext->plain-text-title (cons 'text (cdar stext))))))
      (if parent
          (slot-set! parent 'children
                     (cond
                      ((slot-ref parent 'children)
                       => (lambda (siblings) (append siblings (list node))))
                      (else (list node)))))
      (let loop ((current '()) (input stext))
        (if (null? input)
            (begin
              (slot-set! node 'value (reverse current))
              ;; return the node for the top of the file
              (let find-parent ((node node))
                (let ((parent (slot-ref node 'parent)))
                  (if parent
                      (find-parent parent)
                      node))))
            (if (and (list? (car input)) (name->depth (caar input) max-depth))
                (let ((new-parent node)
                      (new-depth (name->depth (caar input) max-depth)))
                  (do ((i (- depth new-depth) (1- i)))
                      ((< i 0) #f)
                    (set! new-parent (slot-ref new-parent 'parent)))
                  (slot-set! node 'value (reverse current))
                  (stext->help-nodes-helper
                   (cons (cons 'stext (cdar input)) (cdr input))
                   new-parent (name->depth (caar input) max-depth)))
                (loop (cons (car input) current) (cdr input)))))))
  (stext->help-nodes-helper stext #f 0))
